@@ -1,7 +1,7 @@
 /***********************************************************************
- * ratpoints-2.1.2                                                     *
+ * ratpoints-2.2.2                                                     *
  *  - A program to find rational points on hyperelliptic curves        *
- * Copyright (C) 2008, 2009  Michael Stoll                             *
+ * Copyright (C) 2008, 2009, 2022  Michael Stoll                       *
  *                                                                     *
  * This program is free software: you can redistribute it and/or       *
  * modify it under the terms of the GNU General Public License         *
@@ -23,7 +23,7 @@
  *                                                                     *
  * Main program file for the ratpoints executable                      *
  *                                                                     *
- * Michael Stoll, May 27, 2009                                          *
+ * Michael Stoll, May 27, 2009, January 7-18, 2022                     *
  ***********************************************************************/
 
 #include <stdlib.h>
@@ -37,7 +37,7 @@
  **************************************************************************/
 
 #define RATPOINTS_VERSION \
-  "This is ratpoints-2.1.3 Copyright (C) 2008,2009 by Michael Stoll.\n\n" \
+  "This is ratpoints-2.2.2 Copyright (C) 2008,2009,2022,2023 by Michael Stoll.\n\n" \
   "This program comes with ABSOLUTELY NO WARRANTY.\n" \
   "This is free software, and you are welcome to redistribute it under the\n" \
   "terms of the GNU General Public License version 2 or later.\n\n" \
@@ -48,23 +48,21 @@
  **************************************************************************/
 
 mpz_t c[RATPOINTS_MAX_DEGREE+1];  /* The coefficients of f */
-mpz_t tmp, tmp2;
+mpz_t tmp, tmp2;      /* for intermediate results with gmp integers */
 
-int quiet;           /* A flag saying whether to suppress messages */
-int one_point;       /* A flag saying if one point is enough */
-int points_at_infty; /* A flag saying if we should look for points at infty */
-int no_output;       /* A flag that indicates that no points should be 
-                        printed */
-char *print_format;  /* The printf format for printing points */
-char *string_before; /* String to be printed before the points */
-char *string_between;/* String to be printed between the points */
-char *string_after;  /* String to be printed after the points */
+int quiet;            /* A flag saying whether to suppress messages */
+int one_point;        /* A flag saying if one point is enough */
+int points_at_infty;  /* A flag saying if we should look for points at infty */
+int no_output;        /* A flag that indicates that no points should be printed */
+char *print_format;   /* The printf format for printing points */
+char *string_before;  /* String to be printed before the points */
+char *string_between; /* String to be printed between the points */
+char *string_after;   /* String to be printed after the points */
 
 ratpoints_interval domain[2*RATPOINTS_MAX_DEGREE];
-                     /* This contains the intervals representing the
-                        search region */
+                      /* This contains the intervals representing the search region */
 
-char *usage_str = 
+char *usage_str =
     "Usage: ratpoints 'a_0 a_1 ... a_n' max_height\n"
     "                 [-dl low_den] [-du up_den]\n"
     "                 [-f format] [-fs str] [-fm str] [-fe str] [-y] [-Y]\n"
@@ -89,7 +87,8 @@ void error(long);
  * function that processes the points                                     *
  **************************************************************************/
 
-typedef struct {int print_between; int no_output; int one_point; 
+/* structure containing data to be passed to the point-processing function */
+typedef struct {int print_between; int no_output; int one_point;
                 int points_at_infty; int no_y; char *pf;
                 char* string_before; char *string_between;} data;
 
@@ -111,7 +110,7 @@ int process(long a, long b, const mpz_t y, void *info0, int *quit)
           case 'x': printf("%ld", a); break;
           case 'y': if(info->no_y)
                     { putchar('%'); putchar('y'); }
-                    else 
+                    else
                     { mpz_out_str((FILE *)NULL, 10, y); }
                     break;
           case 'z': printf("%ld", b); break;
@@ -145,13 +144,13 @@ int main(int argc, char *argv[])
   ratpoints_args args;
 
   { long n;
-    /* initialise multi-precision integer variables */
+    /* initialize multi-precision integer variables */
     for(n = 0; n <= RATPOINTS_MAX_DEGREE; n++) { mpz_init(c[n]); }
   }
   mpz_init(tmp); mpz_init(tmp2);;
 
   /* read input */
-  if(argc < 3) error(2);
+  if(argc < 3) { error(2); }
   if(read_input(argc-1, &argv[0], &args))
   { message(1, 0, &args);
     return(0);
@@ -162,14 +161,16 @@ int main(int argc, char *argv[])
     if(!(args.flags & RATPOINTS_VERBOSE))
     { message(6, args.height, &args); printf("\n"); }
   }
+  /* use default array size */
   args.array_size = RATPOINTS_ARRAY_SIZE;
 
   if(!quiet && args.num_inter > 0) message(3, 0, &args);
 
-  /* typedef struct {int print_between; int no_output; int one_point; 
+  /* typedef struct {int print_between; int no_output; int one_point;
                      int points_at_infty; int no_y; char *pf;
                      char* string_before; char *string_between;} data; */
 
+  /* initialize data structure to be passed to process... */
   { data *info = malloc(sizeof(data));
 
     info->print_between   = 0;
@@ -181,29 +182,35 @@ int main(int argc, char *argv[])
     info->string_between  = string_between;
     info->no_y            = args.flags & (RATPOINTS_NO_CHECK | RATPOINTS_NO_Y);
 
+    /* ...and call find_points */
     total = find_points(&args, process, (void *)info);
+    /* clean up */
     free(info);
   }
-  if(total < 0)
+  if(total < 0) /* error signaled by find_points? */
   { mpz_clear(tmp); mpz_clear(tmp2);
     if(total == RATPOINTS_NON_SQUAREFREE) error(8);
     error(9);
   }
-  if(total == 0) print_string(string_before);
+  /* if no point was processed, print string_before;
+   * otherwise this is done by process before printing
+   * the first point. */
+  if(total == 0) { print_string(string_before); }
+  /* terminate the output of the points */
   print_string(string_after);
-  if(!quiet) 
+  if(!quiet)
   { if(!(args.flags & RATPOINTS_VERBOSE))
     { printf("\n\n");
       message(4, 0, &args);
       message(7, 0, &args);
       message(8, 0, &args);
     }
-    message(2, total, &args); 
+    message(2, total, &args);
   }
 
+  /* clear multi-precision integer variables */
   mpz_clear(tmp); mpz_clear(tmp2);
   { long n;
-    /* initialise multi-precision integer variables */
     for(n = 0; n <= RATPOINTS_MAX_DEGREE; n++) { mpz_clear(c[n]); }
   }
   return(0);
@@ -219,9 +226,9 @@ int main(int argc, char *argv[])
 
 int read_input(long argc, char *argv[], ratpoints_args *args)
 {
-  int l_seen = 0; /* flag for dealing with -l -u */
-  mpz_t fff;
-  long num_inter = 0;
+  int l_seen = 0;     /* flag for dealing with -l -u */
+  mpz_t fff;          /* temporary variable for gmp integers */
+  long num_inter = 0; /* number of positivity intervals */
 
   /* Initialize args */
   /* typedef struct {mpz_t *cof; long degree; long height;
@@ -229,7 +236,7 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
                     long b_low; long b_high; long sp1; long sp2;
                     double ratio1; double ratio2; long array_size;
                     long sturm; long num_primes; long max_forbidden;
-                    unsigned int flags; ...} 
+                    unsigned int flags; ...}
             ratpoints_args;
   */
   args->cof           = &c[0];
@@ -241,8 +248,8 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
   args->b_high        = -1;
   args->sp1           = -1; /* gives default value */
   args->sp2           = -1; /* gives default value */
-  args->array_size    = RATPOINTS_ARRAY_SIZE;
-  args->sturm         = RATPOINTS_DEFAULT_STURM;
+  args->array_size    = RATPOINTS_ARRAY_SIZE;    /* default */
+  args->sturm         = RATPOINTS_DEFAULT_STURM; /* default */
   args->num_primes    = -1; /* gives default value */
   args->max_forbidden = -1; /* gives default value */
   args->flags         = 0;  /* do the check by default */
@@ -250,16 +257,20 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
                             /* allow reversal of polynomial */
                             /* use Jacobi symbol test */
 
-  mpz_init(fff);
+  mpz_init(fff); /* initialize gmp integer */
+  /* read coefficients */
   { char *s = argv[1];
     long degree = 0;
-    while((degree <= RATPOINTS_MAX_DEGREE) && (s = scan_mpz(s, c[degree]))) 
+
+    while((degree <= RATPOINTS_MAX_DEGREE) && (s = scan_mpz(s, c[degree])))
     { degree++; }
     degree--;
-    if(scan_mpz(s, fff)) error(3);
-    if(degree == 0) error(5);
+    if(scan_mpz(s, fff)) { error(3); }
+    if(degree == 0) { error(5); }
     args->degree = degree;
   }
+
+  /* read height bound */
   if(sscanf(argv[2], " %ld", &(args->height)) != 1 || args->height < 1)
   { error(4); }
   args->b_high = args->height;
@@ -274,64 +285,64 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
   string_between  = NULL;
   string_after    = NULL;
 
-  /* recognise optional args */
+  /* recognize optional args */
   { long i = 3;
     while(i <= argc)
     {
-      if(*(argv[i]) != '-') error(6);
+      if(*(argv[i]) != '-') { error(6); }
       switch(argv[i][1])
       { case 'l': /* lower intevral endpoint */
           if(argc == i) error(7);
           i++;
-          if(l_seen) error(7); /* -l -l */
-          if(num_inter == RATPOINTS_MAX_DEGREE) error(7);
-          if(sscanf(argv[i], " %lf", &domain[num_inter].low) != 1) error(7);
+          if(l_seen) { error(7); } /* -l -l */
+          if(num_inter == RATPOINTS_MAX_DEGREE) { error(7); }
+          if(sscanf(argv[i], " %lf", &domain[num_inter].low) != 1) { error(7); }
           if(num_inter > 0 && domain[num_inter-1].up >= domain[num_inter].low)
-            error(7);
+          { error(7); }
           i++;
           l_seen = 1;
           break;
         case 'u': /* upper interval endpoint */
-          if(argc == i) error(7);
+          if(argc == i) { error(7); }
           i++;
           if(!l_seen)
-          { if(num_inter == 0) domain[0].low = -args->height;
-            else error(7); /* -u -u */
+          { if(num_inter == 0) { domain[0].low = -args->height; }
+            else { error(7); } /* -u -u */
           }
-          if(sscanf(argv[i], " %lf", &domain[num_inter].up) != 1) error(7);
-          if(domain[num_inter].low > domain[num_inter].up) error(7);
+          if(sscanf(argv[i], " %lf", &domain[num_inter].up) != 1) { error(7); }
+          if(domain[num_inter].low > domain[num_inter].up) { error(7); }
           i++;
           l_seen = 0;
           num_inter++;
           break;
         case 'p': /* max number of primes used */
-          if(argc == i) error(6);
+          if(argc == i) { error(6); }
           i++;
-          if(sscanf(argv[i], " %ld", &(args->num_primes)) != 1) error(6);
+          if(sscanf(argv[i], " %ld", &(args->num_primes)) != 1) { error(6); }
           i++;
           break;
         case 'F': /* max number of "forbidden divisors of denominator" */
-          if(argc == i) error(6);
+          if(argc == i) { error(6); }
           i++;
-          if(sscanf(argv[i], " %ld", &(args->max_forbidden)) != 1) error(6);
+          if(sscanf(argv[i], " %ld", &(args->max_forbidden)) != 1) { error(6); }
           i++;
           break;
         case 'n': /* number of primes used for first stage of sieving */
-          if(argc == i) error(6);
+          if(argc == i) { error(6); }
           i++;
-          if(sscanf(argv[i], " %ld", &(args->sp1)) != 1) error(6);
+          if(sscanf(argv[i], " %ld", &(args->sp1)) != 1) { error(6); }
           i++;
           break;
         case 'N': /* number of primes used for sieving altogether */
-          if(argc == i) error(6);
+          if(argc == i) { error(6); }
           i++;
-          if(sscanf(argv[i], " %ld", &(args->sp2)) != 1) error(6);
+          if(sscanf(argv[i], " %ld", &(args->sp2)) != 1) { error(6); }
           i++;
           break;
         case 'f': /* printing format */
           switch(argv[i][2])
           { case 0: /* just -f */
-              if(argc == i) error(6);
+              if(argc == i) { error(6); }
               i++;
               { long l = strlen(argv[i]);
                 print_format = malloc((l+1)*sizeof(char));
@@ -340,7 +351,7 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
               i++;
               break;
             case 's': /* starting string */
-              if(argc == i) error(6);
+              if(argc == i) { error(6); }
               i++;
               { long l = strlen(argv[i]);
                 string_before = malloc((l+1)*sizeof(char));
@@ -349,7 +360,7 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
               i++;
               break;
             case 'm': /* in-between string */
-              if(argc == i) error(6);
+              if(argc == i) { error(6); }
               i++;
               { long l = strlen(argv[i]);
                 string_between = malloc((l+1)*sizeof(char));
@@ -358,7 +369,7 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
               i++;
               break;
             case 'e': /* ending string */
-              if(argc == i) error(6);
+              if(argc == i) { error(6); }
               i++;
               { long l = strlen(argv[i]);
                 string_after = malloc((l+1)*sizeof(char));
@@ -366,7 +377,7 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
               }
               i++;
               break;
-            default: error(6);
+            default: { error(6); }
           }
           break;
         case 'q': /* quiet */
@@ -406,7 +417,7 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
           i++;
           break;
         case 'Y': /* print all points */
-          args->flags |= RATPOINTS_NO_Y;
+          args->flags &= ~RATPOINTS_NO_Y;
           i++;
           break;
         case 'z': /* no output */
@@ -437,28 +448,28 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
           args->sturm = RATPOINTS_DEFAULT_STURM;
           i++;
           if(i <= argc && argv[i][0] != '-')
-          { if(sscanf(argv[i], " %ld", &(args->sturm)) != 1) error(6);
+          { if(sscanf(argv[i], " %ld", &(args->sturm)) != 1) { error(6); }
             i++;
           }
           break;
         case 'd': /* Bounds for denom */
           switch(argv[i][2])
           { case 'l': /* lower bound */
-              if(argc == i) error(6);
+              if(argc == i) { error(6); }
               i++;
-              if(sscanf(argv[i], " %ld", &(args->b_low)) != 1) error(6);
+              if(sscanf(argv[i], " %ld", &(args->b_low)) != 1) { error(6); }
               i++;
               break;
             case 'u': /* upper bound */
-              if(argc == i) error(6);
+              if(argc == i) { error(6); }
               i++;
-              if(sscanf(argv[i], " %ld", &(args->b_high)) != 1) error(6);
+              if(sscanf(argv[i], " %ld", &(args->b_high)) != 1) { error(6); }
               i++;
               break;
-            default: error(6);
+            default: { error(6); }
           }
           break;
-        default: error(6);
+        default: { error(6); }
   } } }
   if(l_seen)
   /* complete last interval */
@@ -471,27 +482,30 @@ int read_input(long argc, char *argv[], ratpoints_args *args)
                      ? "(%x : %z)\n" : "(%x : %y : %z)\n";
   }
 
+  /* quiet implies not verbose */
   if(quiet) { args->flags &= ~RATPOINTS_VERBOSE; }
 
-  mpz_clear(fff);
+  mpz_clear(fff); /* clean up */
   return(0);
 }
 
-/* Read in a long long long integer. */
+/* Read in a long long long integer as an mpz_t. */
 char *scan_mpz(char *s, mpz_t x)
 {
-  long neg = 0;
-  if(s == NULL || *s == 0) return NULL;
-  while(*s == ' ') s++;
-  if(*s == 0) return NULL;
-  if(*s == '-') {neg = 1; s++;}
-  else if(*s == '+') s++;
+  long neg = 0; /* flag for sign */
+
+  if(s == NULL || *s == 0) { return NULL; }
+  while(*s == ' ') { s++; }
+  if(*s == 0) { return NULL; }
+  if(*s == '-') { neg = 1; s++; }
+  else if(*s == '+') { s++; }
   mpz_set_si(tmp2, 0);
   while('0' <= *s && *s <= '9')
   { mpz_mul_ui(tmp2, tmp2, 10);
     mpz_add_ui(tmp2, tmp2, (long)(*s - '0'));
-    s++; }
-  if(neg) mpz_neg(tmp2, tmp2);
+    s++;
+  }
+  if(neg) { mpz_neg(tmp2, tmp2); }
   mpz_set(x, tmp2);
   return s;
 }
@@ -501,11 +515,13 @@ char *scan_mpz(char *s, mpz_t x)
  * output routines                                                        *
  **************************************************************************/
 
+/* Print a polynomial with mpz_t coefficients. */
 void print_poly(mpz_t *coeffs, long degree)
 {
-  int flag = 0;
+  int flag = 0; /* 0 if at beginning */
   int i;
   char *s;
+
   for(i = degree; i >= 0; i--)
   { mpz_set(tmp, coeffs[i]);
     if(mpz_cmp_si(tmp, 0) != 0)
@@ -520,22 +536,16 @@ void print_poly(mpz_t *coeffs, long degree)
       { case 0: s = mpz_get_str((char *) 0, 10, tmp);
                 printf("%s", s); free(s); break;
         case 1: if(mpz_cmp_si(tmp, 1) == 0)
-                {
-                  printf("x");
-                }
+                { printf("x"); }
                 else
-                {
-                  s = mpz_get_str((char *) 0, 10, tmp);
+                { s = mpz_get_str((char *) 0, 10, tmp);
                   printf("%s x", s); free(s);
                 }
                 break;
         default: if(mpz_cmp_si(tmp, 1) == 0)
-                 {
-                   printf("x^%d", i);
-                 }
+                 { printf("x^%d", i); }
                  else
-                 {
-                   s = mpz_get_str((char *) 0, 10, tmp);
+                 { s = mpz_get_str((char *) 0, 10, tmp);
                    printf("%s x^%d", s, i); free(s);
                  }
                  break;
@@ -544,6 +554,7 @@ void print_poly(mpz_t *coeffs, long degree)
   fflush(stdout);
 }
 
+/* Print a string, recognizing \t, \n, \0, \%, \\. */
 void print_string(char *str)
 {
   if(str)
@@ -563,6 +574,7 @@ void print_string(char *str)
 } } }
 
 
+/* Print message number n. */
 void message(long n, long total, ratpoints_args *args)
 {
   switch(n)
@@ -578,7 +590,7 @@ void message(long n, long total, ratpoints_args *args)
     case 3: printf("Search region:\n  ");
             { long i;
               for(i = 0; i < args->num_inter; i++)
-              { if(i) printf(" U ");
+              { if(i) { printf(" U "); }
                 printf("[%f, %f]", args->domain[i].low, args->domain[i].up);
             } }
             printf("\n");
@@ -587,14 +599,14 @@ void message(long n, long total, ratpoints_args *args)
             printf("%ld primes used for both stages of sieving together.\n",
                    args->sp2);
             break;
-    case 5: printf("\nCurve equation is  y^2 = "); 
-            print_poly(args->cof, args->degree); 
-            printf("\n"); 
+    case 5: printf("\nCurve equation is  y^2 = ");
+            print_poly(args->cof, args->degree);
+            printf("\n");
             break;
     case 6: printf("max. Height = %ld\n", args->height);
             break;
     case 7: if(args->flags & RATPOINTS_REVERSED)
-              printf("Polynomial was reversed for better performance.\n");
+            { printf("Polynomial was reversed for better performance.\n"); }
             if(args->flags & RATPOINTS_USE_SQUARES)
             { printf("(Reversed) polynomial is +-monic of odd degree:\n");
               printf("  could restrict denominators to squares.\n");
@@ -604,7 +616,7 @@ void message(long n, long total, ratpoints_args *args)
               printf("  could restrict denominators essentially to squares.\n");
             }
             if(args->flags & RATPOINTS_NO_JACOBI)
-              printf("Jacobi symbol test was not used.\n");
+            { printf("Jacobi symbol test was not used.\n"); }
             if(args->flags & RATPOINTS_NO_CHECK)
             { printf("Points were not checked exactly:\n");
               printf("   some of the printed x-coordinates may not give points.\n");
@@ -614,7 +626,7 @@ void message(long n, long total, ratpoints_args *args)
             { long i;
               for(i = 0; i < args->num_inter; i++)
               { printf("[%lf, %lf]", args->domain[i].low, args->domain[i].up);
-                if(i < args->num_inter -1) printf(" U ");
+                if(i < args->num_inter -1) { printf(" U "); }
               }
               printf("\n");
             }
@@ -623,15 +635,15 @@ void message(long n, long total, ratpoints_args *args)
   fflush(stdout);
 }
 
+/* Print error message number errno and exit. */
 void error(long errno)
 {
   switch(errno)
   { case 3: printf("\nToo many coefficients.\n\n"); break;
     case 4: printf("\nIncorrect height argument.\n\n"); break;
-    case 5: printf("\nThe polynomial must have degree at least 1.\n\n"); break; 
+    case 5: printf("\nThe polynomial must have degree at least 1.\n\n"); break;
     case 6: printf("\nWrong syntax for optional arguments:\n\n");
-    case 2: printf("\n%s\n%s", RATPOINTS_VERSION, usage_str);
-            break;
+    case 2: printf("\n%s\n%s", RATPOINTS_VERSION, usage_str); break;
     case 7: printf("\nIncorrect interval arguments (not alternating, ");
             printf("too many, or not ordered).\n\n");
             break;
